import csv
from django.http import HttpResponse, HttpResponseForbidden
from django.shortcuts import render 
from quotations.models import CustomerQuoteInfo, Quotation
from django.contrib.auth.decorators import login_required
from django.core.paginator import Paginator

from datetime import timedelta 
from django.utils import timezone
from decimal import Decimal
from django.shortcuts import render, redirect, get_object_or_404
from django.http import HttpResponse, HttpResponseForbidden
from django.contrib.auth.decorators import login_required
from django.contrib import messages
import qrcode
from payments.models import Payment
from products.models import Product
from rest_framework import status
from rest_framework.response import Response
from rest_framework.decorators import api_view, permission_classes

from quotations import qr_generator
from quotations.serializer import InquirySerializer 
from customers.models import Customer
from account.models import Account
from config.apipermissions import AllowAllPermissions 
from django.core.files import File
from io import BytesIO

from uom.models import UnitOfMeasure
from fundis.models import Fundi
from colors.models import Color 
from gauge.models import Gauge

from django.core.paginator import Paginator
from django.http import HttpResponse
import csv
from django.core.exceptions import ObjectDoesNotExist

from rest_framework import status

@login_required
def orders(request):
    if request.user.is_admin_user or request.user.is_super_user:
        if request.user.is_admin_user or request.user.is_super_user:
            quotations = Quotation.objects.filter(status="PENDING").order_by('-created_at')  # For admin and super admin, get all quotations
        else:
            # For other users (sales), filter quotations where the seller is the current user
            quotations = Quotation.objects.filter(seller=request.user, status="PENDING").order_by('-created_at')
        
        # Apply filters
        code = request.GET.get('code')
        if code:
            quotations = quotations.filter(qid__icontains=code)
        
        status = request.GET.get('status')
        if status:
            quotations = quotations.filter(status__icontains=status)
        
        customer_name = request.GET.get('customer_name')
        if customer_name:
            quotations = quotations.filter(customer_info__customer_name__icontains=customer_name)
        
        from_date = request.GET.get('from_date')
        to_date = request.GET.get('to_date')
        if from_date and to_date:
            quotations = quotations.filter(created_at__range=[from_date, to_date])
        
        # Pagination
        paginator = Paginator(quotations, 10)  # Show 10 quotations per page
        page_number = request.GET.get('page')
        page_obj = paginator.get_page(page_number)
        
        data = {
            "page_obj": page_obj,
        }
        
        if 'export' in request.GET and request.GET['export'] == 'true':
            response = HttpResponse(content_type='text/csv')
            response['Content-Disposition'] = 'attachment; filename="quotations.csv"'
            
            writer = csv.writer(response)
            writer.writerow(['QUOTATION ID', 'Customer Name', 'Customer Mobile',  'Fundi Name', 'Fundi Number', 'Delivery Location', 'Delivery Mode', 'Payment Terms', 'Lead Source', 'Seller', 'Total Amount', 'Status', 'Created At'])

            from django.core.exceptions import ObjectDoesNotExist

            for quotation in quotations:
                try:
                    customer_quote_info = CustomerQuoteInfo.objects.get(quotation=quotation)
                    customer_name = customer_quote_info.customer.customer_name if customer_quote_info.customer else ""
                    customer_mobile = customer_quote_info.customer.customer_mobile if customer_quote_info.customer else ""
                    fundi = customer_quote_info.customer.fundi if (customer_quote_info.customer and customer_quote_info.customer.fundi) else ""
                    fundi_details = customer_quote_info.customer.fundi_details if (customer_quote_info.customer and customer_quote_info.customer.fundi_details) else ""
                except ObjectDoesNotExist:
                    # If CustomerQuoteInfo does not exist, set its info and customer's info to ""
                    customer_quote_info = CustomerQuoteInfo()
                    customer_name = ""
                    customer_mobile = ""
                    fundi = ""
                    fundi_details = ""

                writer.writerow([
                    quotation.qid, 
                    customer_name,
                    customer_mobile, 
                    fundi,
                    fundi_details,
                    customer_quote_info.delivery_location if customer_quote_info.delivery_location else "",  
                    customer_quote_info.delivery_mode if customer_quote_info.delivery_mode else "",   
                    customer_quote_info.payment_terms if customer_quote_info.payment_terms else "",   
                    customer_quote_info.lead_source if customer_quote_info.lead_source else "",  
                    customer_quote_info.seller.email if customer_quote_info.seller else "",   
                    quotation.total_amount, 
                    quotation.status, 
                    quotation.created_at
                ])

            return response
    
        return render(request, "backoffice/orders/orders.html", data)
    else:
        return HttpResponseForbidden("You are not authorized to view this page.")
